%% read income data
% setup the import options
opts = delimitedTextImportOptions("NumVariables", 1);

% specify range and delimiter
opts.DataLines = [1, Inf];
opts.Delimiter = ",";

% specify column names and types
opts.VariableNames = "ly";
opts.VariableTypes = "double";

% specify file level properties
opts.ExtraColumnsRule = "ignore";
opts.EmptyLineRule = "read";

% import the data
for year = 1985:2016
    tbl = readtable(['..\out\ly\ly_' int2str(year) '.csv'], opts);
    ly{year-1984} = tbl.ly;
end

% Clear temporary variables
clear opts tbl year
%% create quantiles, calculate cumulative income

% intialize quantile vectors
p50 = zeros(1,length(ly));
p90 = zeros(1,length(ly));
p91 = zeros(1,length(ly));
p92 = zeros(1,length(ly));
p92_5 = zeros(1,length(ly));
p95 = zeros(1,length(ly));
p97_5 = zeros(1,length(ly));
p98_5 = zeros(1,length(ly));
p99 = zeros(1,length(ly));
p99_5 = zeros(1,length(ly));
p99_9 = zeros(1,length(ly));

% uses parfor to significantly speed up computation on multi-core processors; on single core use for loops instead
parfor i = 1:length(ly)
    % sort earnings in increasing order
    ly{i} = sort(ly{i}); 
    
    % quantiles
    p50(i) = quantile(ly{i},0.5);
    p90(i) = quantile(ly{i},0.9); % quantiles...
    p91(i) = quantile(ly{i},0.91);
    p92_5(i) = quantile(ly{i},0.925);
    p95(i) = quantile(ly{i},0.95);
    p97_5(i) = quantile(ly{i},0.975);
    p98_5(i) = quantile(ly{i},0.985);
    p99(i) = quantile(ly{i},0.99); 
    p99_5(i) = quantile(ly{i},0.995);
    p99_9(i) = quantile(ly{i},0.999);
    
    n(i) = length(ly{i}); % num of observations
    m(i) = sum(ly{i}>p99(i)); % number of observations top coded
    
    q_all{i} = linspace(1,0,length(ly{i}))'; % construct (complementary) quantiles
    
    % produce a truncated sample to deal with top coding
    ly_tc{i} = ly{i}(ly{i}>p90(i));
    ly_tc{i}(ly_tc{i}>p99(i)) = p99(i); % impose topcoding at 99th percentile
    n_tc(i) = length(ly_tc{i}); % number of observations in the truncated sample

    % earnings of the bottom
    bottom90(i) = sum(exp(ly{i}(ly{i}<p90(i)))); 
    bottom95(i) = sum(exp(ly{i}(ly{i}<p95(i)))); 
    bottom99(i) = sum(exp(ly{i}(ly{i}<p99(i)))); 
    
    % earnings of the top
    unadjusted_top01(i) = sum(exp(ly{i}(ly{i}>=p99_9(i))));
    unadjusted_top05(i) = sum(exp(ly{i}(ly{i}>=p99_5(i))));
    unadjusted_top1(i) = sum(exp(ly{i}(ly{i}>=p99(i))));
    unadjusted_top5(i) = sum(exp(ly{i}(ly{i}>=p95(i))));
    unadjusted_top10(i) = sum(exp(ly{i}(ly{i}>=p90(i))));
    unadjusted_total(i) = sum(exp(ly{i}));
end

%% estimate using 4 methods
% method 1: mle
for i = 1:length(ly)
    a_ml(i) = (n_tc(i)-m(i))/(sum(ly_tc{i}-p90(i)));
end


% method 2: probability integral transform
t = 0.5;
parfor i = 1:length(ly)
    % bisection method to solve nonlinear equation
    a_low = 1;
    a_high = 5;
    for j = 1:12
        a_temp = (a_low+a_high)/2;
        y = (1+t)*(exp(ly_tc{i}-p90(i))).^(-a_temp*t)/(1 + t*(exp(p99(i)-p90(i)))^(-a_temp*(1+t)));
        if mean(y)>1
            a_low = a_temp;
        else
            a_high = a_temp;
        end
    end
    a_pits(i) = (a_low+a_high)/2;
end

% method 3: regression on quantiles
q_vec = linspace(0.9,0.985,100)';
for i = 1:length(ly)
    x_q = quantile(ly{i},q_vec);
    b = [ones(length(q_vec),1), -(x_q)]\log(1-q_vec);    
    a_q(i) = b(2);
end

% method 4: regression on kernel pdf
parfor i = 1:length(ly)
    x_vec = linspace(p90(i),p98_5(i),100)';
    pd_kernel_log = fitdist(ly{i},'kernel','kernel','epanechnikov');
    pd_kernel_level = fitdist(exp(ly{i}),'kernel','kernel','epanechnikov');

    pdf_kernel_log = pdf(pd_kernel_log,x_vec);
    pdf_kernel_level = pdf(pd_kernel_level,exp(x_vec));
    
    b_log = [ones(size(x_vec)),...
        -(x_vec)]\log(pdf_kernel_log);
    b_level = [ones(size(x_vec)),...
        -(x_vec)]\log(pdf_kernel_level);
    a_kernel_log(i) = b_log(2); % note: no need to remove 1, because density is of log(y)
    a_kernel_level(i) = b_level(2)-1; % note: no need to remove 1, because density is of log(y)
end
a_all = [a_ml; a_pits; a_q; a_kernel_log];

%% income shares, based on Pareto-tail assumption

top01 = 0.001*n.*exp(p99).*(0.1).^(-1./a_ml).*a_ml./(a_ml-1);
top05 = 0.005*n.*exp(p99).*(0.5).^(-1./a_ml).*a_ml./(a_ml-1);
top1 = 0.01*n.*exp(p99).*a_ml./(a_ml-1);
top5 = bottom99 - bottom95 + top1;
top10 = bottom99 - bottom90 + top1;


top10_share = top10./(top10+bottom90);
top5_share = top5./(top10+bottom90);
top1_share = top1./(top10+bottom90);
top05_share = top05./(top10+bottom90);
top01_share = top01./(top10+bottom90);


%% plot income shares in two panels (left out of paper)
cmap = lines(4);
figure('DefaultTextFontName','Times','DefaultAxesFontName','Times',...
    'Units','Inches','Position',[3 3 8 4.5]); 
tiledlayout(1,2,'TileSpacing','compact');

% left panel: top 10 and 5 percent
ax = nexttile;
plt = plot(1985:2016,100*[unadjusted_top10'./unadjusted_total' top10_share' unadjusted_top5'./unadjusted_total' top5_share'],'LineWidth',1); 
plt(2).Color = cmap(1,:);
plt(1).LineStyle = '--';
plt(2).Marker = 's';
plt(3).Color = cmap(2,:);
plt(3).LineStyle = '--';
plt(4).Color = cmap(2,:);
plt(4).Marker = 'o';
legend('unadj. top 10%','adj. top 10%','unadj. top 5%','adj. top 5%','Location','SouthOutside','orientation','horizontal','numColumns',2);
xlim([1985 2016]); ylim([10 35]); ylabel('share of income'); 
ax.YGrid = 'on';grid on;

% right panel: top 1 and 0.1 percent
ax = nexttile;
plt = plot(1985:2016,100*[unadjusted_top1'./unadjusted_total' top1_share' unadjusted_top01'./unadjusted_total' top01_share'],'LineWidth',1); 
plt(1).Color = cmap(1,:);
plt(2).Color = cmap(1,:);
plt(1).LineStyle = '--';
plt(2).Marker = 's';
plt(3).Color = cmap(2,:);
plt(4).Color = cmap(2,:);
plt(3).LineStyle = '--';
plt(4).Marker = 'o';
box on;

legend('unadj. top 1%','adj. top 1%','unadj. top 0.1%','adj. top 0.1%','Location','SouthOutside','orientation','horizontal','numColumns',2);
xlim([1985 2016]); ylim([0 9]); ylabel('share of income'); 
ax.YGrid = 'on'; grid on;


%% Figure 5: plot percent growth in income shares - separate figures for unadjusted and adjusted shares
cmap = lines(5);
cmap(3:4,:) = [];
cmap(end+1,:) = [0 0 0];
figure('DefaultTextFontName','Times','DefaultAxesFontName','Times'); 
set(gcf,'Units','Inches','Position',[4 4 4 3]);

temp1 = [unadjusted_top10'./unadjusted_total' unadjusted_top5'./unadjusted_total' unadjusted_top1'./unadjusted_total' unadjusted_top01'./unadjusted_total'];
temp1 = 100*(temp1./temp1(1,:) - 1);
plt = plot(1985:2016,temp1,'LineWidth',1); 
plt(1).Color = cmap(1,:);
plt(1).Marker = 'o';
plt(2).Color = cmap(2,:);
plt(2).Marker = 's';
plt(2).LineStyle = '--';
plt(3).Color = cmap(4,:);
plt(3).Marker = 'd';
plt(4).Color = cmap(3,:);
plt(4).Marker = '+';
plt(4).LineStyle = '--';
legend('top 10%','top 5%','top 1%','top 0.1%','Location','NorthEast','orientation','horizontal','numColumns',2);
legend('boxoff');
xlim([1985 2016]); ylim([0 70]); ylabel('percent growth in income share'); 
grid on;
if print_graphs == 1
    %print -dpdf -fillpage ../figs/income_shares_twopanels.pdf
    exportgraphics(gcf, "../figs/income_shares_growth1.pdf")
end


figure('DefaultTextFontName','Times','DefaultAxesFontName','Times'); 
set(gcf,'Units','Inches','Position',[4 4 4 3]);
temp2 = [top10_share' top5_share' top1_share' top01_share'];
temp2 = 100*(temp2./temp2(1,:) - 1);
plt = plot(1985:2016,temp2,'LineWidth',1); 
plt(1).Color = cmap(1,:);
plt(1).Marker = 'o';
plt(2).Color = cmap(2,:);
plt(2).Marker = 's';
plt(2).LineStyle = '--';
plt(3).Color = cmap(4,:);
plt(3).Marker = 'd';
plt(4).Color = cmap(3,:);
plt(4).Marker = '+';
plt(4).LineStyle = '--';
box on;

legend('top 10%','top 5%','top 1%','top 0.1%','Location','SouthEast','orientation','horizontal','numColumns',2);
legend('boxoff');

xlim([1985 2016]); ylim([0 70]); ylabel('percent growth in income share'); 
ax.YGrid = 'on'; grid on;

%% Figure A.3: the tails of the distribution are approximately Pareto
figure;
tiledlayout(4,1,'TileSpacing','compact');
for i = 1:10:length(ly)
    nexttile;
    % plot only the raw data
    plot((ly{i}(q_all{i}>0.01 & q_all{i}<0.10)), log(q_all{i}(q_all{i}>0.01 & q_all{i}<0.10)));
     
    title(int2str(1985-1+i));
    grid on; 
    axis('tight');
    ylabel('log (1-F(y))');
end
xlabel('log(y)');

%% Same graph as A.3 but using logarithmic scale instead of logged variables (left out of the paper)
figure;
tiledlayout(4,1,'TileSpacing','compact');
for i = 1:10:length(ly)
    nexttile;
    loglog(exp(ly{i}(q_all{i}>0.01 & q_all{i}<0.10))/1000, (q_all{i}(q_all{i}>0.01 & q_all{i}<0.10)));
    title(int2str(1985-1+i));
    grid on; ylim([0.01 0.1]);
    ylabel('1-F(y)');
end
xlabel('y (thousand euros)');


%% show how this works: compare regression on kernel density to regression on quantiles
x_vec = linspace(p90(end),p98_5(end),100)';
pd_kernel_log = fitdist(ly{end},'kernel','kernel','epanechnikov');
pdf_kernel_log = pdf(pd_kernel_log,x_vec);
figure; subplot(121); plot(x_vec,[log(pdf_kernel_log)],'LineWidth',2); title('kernel density');
subplot(122); plot(x_q,log(1-q_vec),'LineWidth',2); title('quantiles');

%% standard errors, etither using bootstrap or simulation
N = 100; % number of bootstrapped samples / simulations

% matrices to store estimates
a_ml_sim = zeros(N,length(a_ml));
a_q_sim = zeros(N,length(a_ml));
a_pits_sim = zeros(N,length(a_ml));
a_kernel_log_sim = zeros(N,length(a_ml));

bootstrap_type = 1; % nonparametric
rng(0); % keeps same pseudorandom sequence

parfor i = 1:length(a_ml) % note: parfor introduces some randomness
    disp(['year = ' int2str(i-1+1985)]);
    for j = 1:N
        
        % nonparametric bootstrap
        if bootstrap_type==1
            x = randsample(ly{i},n(i),true);
            x = sort(x);
        end
        
        % parametric simulation
        if bootstrap_type==2
            x = sort(rand(n(i),1));
            x = (p90(i))- (log(x)-log(0.1))/a_ml(i);
        end
        p99_temp = quantile(x,0.99);
        
        x_tc = x(x>p90(i));
        x_tc(x_tc>p99_temp) = p99_temp;
        n_tc_temp = length(x_tc);
        
        % mle estimator
        a_ml_sim(j,i) = (n_tc_temp-sum(x>=(p99_temp)))/sum((x_tc)-p90(i));
        
        % prob integral method
        a_low = 1;
        a_high = 4;
        
        for k = 1:12
            a_temp = (a_low+a_high)/2;
            y = (1+t)*(exp(x_tc)/exp(p90(i))).^(-a_temp*t)/(1 + t*(exp(p99_temp-p90(i)))^(-a_temp*(1+t)));
            if mean(y)>1
                a_low = a_temp;
            else
                a_high = a_temp;
            end
        end
        a_pits_sim(j,i) = (a_low+a_high)/2;
        
        % quantiles
        x_q = quantile(x,q_vec);
        b = [ones(length(q_vec),1), -(x_q)]\log(1-q_vec);
        a_q_sim(j,i) = b(2);
        
        % kernel
        xx = quantile(x,[0.9 0.985]);
        x_vec = linspace((xx(1)),(xx(2)),100)';
        pd_kernel_log = fitdist((x),'kernel','kernel','epanechnikov','bandwidth',0.05);
        pdf_kernel_log = pdf(pd_kernel_log,x_vec);
        b_log = [ones(size(x_vec)),...
            -(x_vec)]\log(pdf_kernel_log);
        a_kernel_log_sim(j,i) = b_log(2); % note: no need to remove 1, because density is of log(y)        
    end
end
% get standard errors from bootstrap/simulation
std_a_ml = std(a_ml_sim);
std_a_q = std(a_q_sim);
std_a_pits = std(a_pits_sim);
std_a_kernel_log = std(a_kernel_log_sim);
std_a_all = [std_a_ml; std_a_pits; std_a_q; std_a_kernel_log];

%% Figure A.4: bootstrapped distribution for 2016
figure; up_ = 4; down_ = -4;
bins = a_ml(end)+linspace(down_,up_,20)*std_a_kernel_log(end);
subplot(222); hist(a_q_sim(:,end),bins); 
xlim(a_ml(end)+[down_,up_]*std_a_kernel_log(end)); ylim([0 40]);
title('(b) QS');
subplot(224); hist(a_pits_sim(:,end),bins);
xlim(a_ml(end)+[down_,up_]*std_a_kernel_log(end)); ylim([0 40]);
title('(d) PITS'); 
subplot(221); hist(a_ml_sim(:,end),bins);
xlim(a_ml(end)+[down_,up_]*std_a_kernel_log(end)); ylim([0 40]);
title('(a) ML'); 
subplot(223); hist(a_kernel_log_sim(:,end),bins);
xlim(a_ml(end)+[down_,up_]*std_a_kernel_log(end)); ylim([0 40]);
title('(c) KDS');

%% Figure A.5: Pareto-tail indexes with confidence band
figure;
plt = plot(1985:2016, a_all','LineWidth',2);
for i = 1:length(plt)
    patch([1985:2016 2016:-1:1985], [a_all(i,:)+std_a_all(i,:)*2 fliplr(a_all(i,:)-std_a_all(i,:)*2)],plt(i).Color,'FaceAlph',0.5,'EdgeColor','none')
end
xlim([1985 2016]);
ylabel('Pareto-tail index'); legend('ML','PITS','QS','KDS'); grid on;
